package com.amqp.shanxi.util;

import sun.misc.BASE64Decoder;
import sun.misc.BASE64Encoder;

import javax.crypto.BadPaddingException;
import javax.crypto.Cipher;
import javax.crypto.IllegalBlockSizeException;
import javax.crypto.NoSuchPaddingException;
import javax.crypto.spec.IvParameterSpec;
import javax.crypto.spec.SecretKeySpec;

import java.io.IOException;
import java.security.InvalidAlgorithmParameterException;
import java.security.InvalidKeyException;
import java.security.NoSuchAlgorithmException;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

/**
 *  解密工具类
 * @author z0990
 */
public class AESUtil {

    /**
     * 用于存储已生成的Cipher
     */
    private static Map<String, Cipher> ciperCache = new ConcurrentHashMap<>();

    /**
     * AES解密
     * @author z0990
     * @param content 待解密的内容
     * @param decryptKey 解密秘钥
     * @param iv 向量
     * @return 解密数据
     */
    public static byte[] decrypt(String content, String decryptKey, String iv) throws NoSuchAlgorithmException, NoSuchPaddingException, InvalidAlgorithmParameterException, InvalidKeyException, IOException, BadPaddingException, IllegalBlockSizeException {
        Cipher cipher = fetchDecryptCipher(decryptKey, iv);

        BASE64Decoder decoder = new BASE64Decoder();
        return cipher.doFinal(decoder.decodeBuffer(content));
    }

    /**
     * AES加密
     * @param data 待加密的内容
     * @param key 解密秘钥
     * @param iv 向量
     * @return 加密并经base64编码后的数据
     */
    public static String encrypt(String data, String key, String iv) throws InvalidAlgorithmParameterException, NoSuchAlgorithmException, InvalidKeyException, NoSuchPaddingException, IOException, BadPaddingException, IllegalBlockSizeException {
        Cipher cipher = getEncryptCipher(key, iv);
        byte[] bytes = cipher.doFinal(data.getBytes(Constants.CHARSET_UTF8));
        BASE64Encoder encoder = new BASE64Encoder();
        return encoder.encode(bytes);
    }

    private static Cipher getEncryptCipher(String key, String iv) throws NoSuchAlgorithmException, NoSuchPaddingException, InvalidAlgorithmParameterException, InvalidKeyException, IOException {
        Cipher cipher = ciperCache.get(key);
        if (cipher == null) {
            cipher = fetchEncryptCipher(key, iv);
            ciperCache.put(key, cipher);
        }

        return cipher;
    }

    private static Cipher fetchDecryptCipher(String decryptKey, String iv) throws InvalidAlgorithmParameterException, NoSuchAlgorithmException, InvalidKeyException, NoSuchPaddingException, IOException {
        return createCipher(decryptKey, iv, Cipher.DECRYPT_MODE);
    }

    private static Cipher fetchEncryptCipher(String encryptKey, String iv) throws InvalidAlgorithmParameterException, NoSuchAlgorithmException, InvalidKeyException, NoSuchPaddingException, IOException {
        return createCipher(encryptKey, iv, Cipher.ENCRYPT_MODE);
    }

    private static Cipher createCipher(String key, String iv, int mode) throws NoSuchPaddingException, NoSuchAlgorithmException, IOException, InvalidAlgorithmParameterException, InvalidKeyException {
        Cipher cipher = Cipher.getInstance(Constants.DECRYPT_CIPHER_ALGORITHM);
        BASE64Decoder decoder = new BASE64Decoder();

        SecretKeySpec keySpec = new SecretKeySpec(decoder.decodeBuffer(key), Constants.DECRYPT_ALGORITHM);
        IvParameterSpec ivSpec = new IvParameterSpec(decoder.decodeBuffer(iv));

        //初始化
        cipher.init(mode, keySpec, ivSpec);

        return cipher;
    }
}
